@@ -24,6 +24,9 @@
 #include <gst/gst.h>
 #include <gst/base/gstpushsrc.h>
 
+#include <gst/video/gstvideometa.h>
+#include <gst/video/gstvideopool.h>
+
 G_BEGIN_DECLS
 
 #define GST_TYPE_VIDEO_TEST_SRC \
@@ -103,18 +106,6 @@ typedef enum {
   GST_VIDEO_TEST_SRC_BAR
 } GstVideoTestSrcPattern;
 
-/**
- * GstVideoTestSrcColorSpec:
- * @GST_VIDEO_TEST_SRC_BT601: ITU-R Rec. BT.601/BT.470 (SD)
- * @GST_VIDEO_TEST_SRC_BT709: ITU-R Rec. BT.709 (HD)
- *
- * The color specification to use.
- */
-typedef enum {
-  GST_VIDEO_TEST_SRC_BT601,
-  GST_VIDEO_TEST_SRC_BT709
-} GstVideoTestSrcColorSpec;
-
 typedef struct _GstVideoTestSrc GstVideoTestSrc;
 typedef struct _GstVideoTestSrcClass GstVideoTestSrcClass;
 
@@ -131,23 +122,16 @@ struct _GstVideoTestSrc {
   /* type of output */
   GstVideoTestSrcPattern pattern_type;
 
-  /* Color spec of output */
-  GstVideoTestSrcColorSpec color_spec;
-
   /* video state */
-  char *format_name;
-  gint width;
-  gint height;
-  struct fourcc_list_struct *fourcc;
-  gint bpp;
-  gint rate_numerator;
-  gint rate_denominator;
+  GstVideoInfo info;
+  gboolean bayer;
+  gint x_invert;
+  gint y_invert;
 
   /* private */
   gint64 timestamp_offset;              /* base offset */
   GstClockTime running_time;            /* total running time */
   gint64 n_frames;                      /* total frames sent */
-  gboolean peer_alloc;
 
   /* zoneplate */
   gint k0;
@@ -171,12 +155,13 @@ struct _GstVideoTestSrc {
   gint horizontal_offset;
   gint horizontal_speed;
 
-  void (*make_image) (GstVideoTestSrc *v, unsigned char *dest, int w, int h);
+  void (*make_image) (GstVideoTestSrc *v, GstVideoFrame *frame);
 
   /* temporary AYUV/ARGB scanline */
   guint8 *tmpline_u8;
   guint8 *tmpline;
   guint8 *tmpline2;
+  guint16 *tmpline_u16;
 };
 
 struct _GstVideoTestSrcClass {
